/****************************************************************************
*  This file is part of the blinky program for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  20.09.2009  mifi   First Version, tested with a LPC-P2148 board
****************************************************************************/
#define __HARDWARE_C__

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include "arch.h"
#include "hardware.h"
#include "tn.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
#define TIMER0_ID 4

/*
 * The LED1 is connected to port P0.10.
 * If the LED on your eval board is connected to an 
 * other port, you must change the defines here.
 */
#define LED1      (1<<10)

#define LED_DIR   IO0DIR

#define LED_ON    IO0CLR
#define LED_OFF   IO0SET

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  Timer0IRQHandler                                                       */
/*                                                                         */
/*  For the purpose of calculating timeouts and delays, TNKernel uses a    */
/*  time tick timer. In TNKernel, this time tick timer must to be a some   */
/*  kind of hardware timer that produces interrupt for time ticks          */
/*  processing. The period of this timer is user determined (usually in    */
/*  the range 0.5...20 ms). Within the time ticks interrupt processing,    */
/*  it is only necessary to call the tn_tick_int_processing() functions    */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void Timer0IRQHandler (void)
{
   /* Clear timer interrupt */
   T0IR = 1;
   
   tn_tick_int_processing();
} /* Timer0IRQHandler */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   IO0DIR |= LED1;

   LedOff();
} /* LEDInit */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  Init the hardware like the timer and the GPIO port for the LED.        */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void HardwareInit (void)
{
   /*
    * Init the interrupt system first
    */            
   tn_irq_init();

   /*
    * The CPU clock is set to 60MHz and the APB bus to 30MHz
    * by the startup code from CrossWorks for ARM. Here we must 
    * only init the timer. Setup the Timer0 with a period of 1ms.
    */

   /* Disable Timer0 */
   T0TCR = 0;
   
   /* 
    * Set the prescale counter.
    * This causes the TC to increment on every PCLK when PR = 0
    */
   T0PR = 0;

   /*
    * Assume a CPU clock of 60MHz and a APB bus clock of 30MHz.
    * The timer tick will be set to 1ms (30MHz / 1KHz = 30000)
    */
   T0MR0 = (30000-1);
   
   /*
    * Interrupt on MR0, an interrupt is generated when MR0 matches 
    * the value in the TC.
    *
    * Reset on MR0: the TC will be reset if MR0 matches it.
    */
   T0MCR = (T0MCR_MR0I_MASK | T0MCR_MR0R_MASK);

   /* Enable the Timer0 */   
	T0TCR = 1;

   /* Install timer0 with prio 1 */
   tn_irq_install(TIMER0_ID, 1, Timer0IRQHandler);
   
   
   /*
    * Init the GPIO for the LED
    */
   LEDInit();

} /* HardwareInit */

/***************************************************************************/
/*  tn_cpu_int_enable                                                      */
/*                                                                         */
/*  The tn_cpu_int_enable() function enables all interrupts for vectors    */
/*  utilized by the user project and than enables global interrupts.       */
/*  The user must enable the system time ticks interrupt in this function. */
/*  TNKernel calls this function without user intervention.                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_cpu_int_enable (void)
{
   /* Enable Timer0 interrupt */
   VICIntEnable = (1<<TIMER0_ID);
   
   tn_arm_enable_interrupts();
} /* tn_cpu_int_enable */

/***************************************************************************/
/*  LedOn                                                                  */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void LedOn (void)
{
   LED_ON |= LED1;
} /* LedOn */

/***************************************************************************/
/*  LedOff                                                                 */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void LedOff (void)
{
   LED_OFF |= LED1;
} /* LedOff */

/*** EOF ***/
