/****************************************************************************
*  This file is part of the blinky program for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  19.09.2009  mifi   First Version
****************************************************************************/
#define __MAIN_C__

/*
 * This file of the blinky application is hardware independent.
 * All the stuff for initialisation of the hardware can
 * be found in hardware.c
 */

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include "hardware.h"
#include "tn.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
/*
 * Priority and stack size of the LED task.
 */
#define TASK_LED_PRIORITY  10
#define TASK_LED_STK_SIZE  128

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/

/* 
 * Some TASK variables like stack and task control block.
 */
static unsigned int TaskLedStack[TASK_LED_STK_SIZE];
static TN_TCB       TaskLedTCB;

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  LEDTask                                                                */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static void LEDTask (void *pParam)
{
   unsigned short Blink = 1;

   /* Prevent compiler warning */
   pParam = pParam;
   
   while(1)
   {
      if (Blink & 1)
      {
         LedOn();
      }
      else
      {
         LedOff();
      }   
      
      Blink = Blink ^ 1;
      
      /* Sleep 100 ticks */
      tn_task_sleep(100);
   }
} /* LEDTask */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  main                                                                   */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
int main (void)
{
   /*
    * Before the application which is using TNKernel can be started, 
    * the hardware of the target must be initialized. For the purpose of 
    * calculating timeouts and delays, TNKernel uses a time tick timer. 
    * In TNKernel, this time tick timer must to be a some kind of hardware 
    * timer that produces interrupt for time ticks processing. This small 
    * application here will blink with a LED, therefore the GPIO system 
    * must be initialized too. All this will be done by the HardwareInit
    * function.
    */
   HardwareInit();

   /*
    * The tn_start_system() function performs all actions to initialize and 
    * start TNKernel (initializes the system global variables, creates tasks 
    * with priorities 0 and 31, calls start-up functions etc.) 
    * The tn_start_system() function internally calls the tn_app_init() function.
    */
   tn_start_system();    
  
   /*
    * This return here make no sense. But to prevent the compiler warning:
    * "return type of 'main' is not 'int'
    * An int as return is used :-)
    */ 
   return(0);
} /* main */

/***************************************************************************/
/*  tn_app_init                                                            */
/*                                                                         */
/*  The contents of this function has to be defined by the user and may be */
/*  empty. In this function the user has to create all tasks, semaphores,  */
/*  data queues, memory pools etc., which he wants to have before the      */
/*  system starts.                                                         */
/*                                                                         */
/*  Note: During this function the interrupts are disabled,                */ 
/*        and will be enabled later by TNKernel.                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_app_init (void)
{
   TaskLedTCB.id_task = 0;
   tn_task_create(&TaskLedTCB,               /* task TCB */
                 LEDTask,                    /* task function */
                 TASK_LED_PRIORITY,          /* task priority */
                 &(TaskLedStack              /* task stack first addr in memory */
                   [TASK_LED_STK_SIZE-1]),
                 TASK_LED_STK_SIZE,          /* task stack size (in int,not bytes) */
                 NULL,                       /* task function parameter */
                 TN_TASK_START_ON_CREATION   /* Creation option */
                 );
} /* tn_app_init */

/*** EOF ***/
