/****************************************************************************
*  This file is part of the blinky program for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  19.09.2009  mifi   First Version, tested with a AT91SAM7X-EK board
****************************************************************************/
#define __HARDWARE_C__

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include "arch.h"
#include "hardware.h"
#include "tn.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*
 * The LED1 is connected to port PB19.
 * If the LED on your eval board is connected to an 
 * other port, you must change the defines here.
 */
#define LED1      (1<<19)

#define LED_PER   PIOB_PER
#define LED_OER   PIOB_OER
#define LED_IDR   PIOB_IDR

#define LED_ON    PIOB_CODR
#define LED_OFF   PIOB_SODR

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  SystemIRQHandler                                                       */
/*                                                                         */
/*  For the purpose of calculating timeouts and delays, TNKernel uses a    */
/*  time tick timer. In TNKernel, this time tick timer must to be a some   */
/*  kind of hardware timer that produces interrupt for time ticks          */
/*  processing. The period of this timer is user determined (usually in    */
/*  the range 0.5...20 ms). Within the time ticks interrupt processing,    */
/*  it is only necessary to call the tn_tick_int_processing() functions    */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void SystemIRQHandler (void)
{
   volatile int data;

   if (PIT_SR & 1)               /* Check PIT Interrupt  */
   {
      data = PIT_PIVR;           /* Clear PIT int source */
      tn_tick_int_processing();
   }
} /* SystemIRQHandler */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   LED_PER = LED1;
   LED_OER = LED1;
   LED_IDR = LED1;

   LedOff();
} /* LEDInit */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  Init the hardware like the time and the GPIO port for the LED.         */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void HardwareInit (void)
{
   /*
    * Init the interrupt system first
    */            
   tn_irq_init();

   /*
    * The CPU clock is set to 48MHz by the startup code
    * from CrossWorks for ARM. Here we must only init the
    * timer. Setup the Periodic Interval Timer with a period
    * of 1ms.
    */
   PMC_PCER |= (1<<2);        /* Enable PIO clock */
   PIT_MR    = (1 << 24) |    /* PIT Enable */
               (1 << 25) |    /* PIT Interrupt Enable */
               (3000*1-1);    /* Periodic Interval Value - 1 ms */

   /*
    * Install the SystemIRQHandler, which will be used
    * for the timer interrupt.
    *
    * Setup System Interrupt Mode and Vector with Priority 7
    * PIT's int is System Interrupt, Peripheral ID for PIT is 1
    */
   tn_irq_install(1, ((1 << 5) | 7), SystemIRQHandler);

            
   /*
    * Init the GPIO for the LED
    */
   LEDInit();
   
} /* HardwareInit */

/***************************************************************************/
/*  tn_cpu_int_enable                                                      */
/*                                                                         */
/*  The tn_cpu_int_enable() function enables all interrupts for vectors    */
/*  utilized by the user project and than enables global interrupts.       */
/*  The user must enable the system time ticks interrupt in this function. */
/*  TNKernel calls this function without user intervention.                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_cpu_int_enable (void)
{
   /*
    * Enable System Peripheral Int
    */
   AIC_IECR = (1<<1);

   tn_arm_enable_interrupts();
} /* tn_cpu_int_enable */

/***************************************************************************/
/*  LedOn                                                                  */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void LedOn (void)
{
   LED_ON = LED1;
} /* LedOn */

/***************************************************************************/
/*  LedOff                                                                 */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void LedOff (void)
{
   LED_OFF = LED1;
} /* LedOff */

/*** EOF ***/
