/****************************************************************************
*  This file is part of the stm32 port for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*  History:
*
*  26.09.2009  mifi   First Version
****************************************************************************/
#if defined(TN_ARCH_CM3) && defined(TN_PORT_STM32)

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include <targets/stm32f10x.h>
#include "tn.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
typedef void (*IRQ_FUNC)(void);

/*=========================================================================*/
/*  DEFINE: Definition of all public Data                                  */
/*=========================================================================*/
IRQ_FUNC IRQTable[TN_MAX_IRQ_ID];

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static unsigned long *InterruptPriorityRegistersTable = (unsigned long*)(0xE000E400);

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  DefaultIRQHandler                                                      */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static void DefaultIRQHandler (void)
{
   volatile int i = 0;
   
   while(1)
   {
      i++;
   }
} /* DefaultIRQHandler */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/

/***************************************************************************/
/*  tn_inside_int                                                          */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: 1 or 0                                                         */
/***************************************************************************/
int tn_inside_int (void)
{
   if(SCB_ICSR & SCB_ICSR_VECTACTIVE_MASK)
   {
      return 1;
   }
   else
   {   
      return 0;
   } 
} /* tn_inside_int */

/***************************************************************************/
/*  tn_irq_init                                                            */
/*                                                                         */
/*  Here the irq system of the cpu will be init.                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_irq_init (void)
{
   int i;
   
   /* Disable cpu interrupt */
   tn_arm_disable_interrupts();
   
   /* Set IRQTable to 0 */
   for(i=0; i<TN_MAX_IRQ_ID; i++)
   {
      IRQTable[i] = DefaultIRQHandler;
   }
   
} /* tn_irq_init */

/***************************************************************************/
/*  tn_irq_install                                                         */
/*                                                                         */
/*  In    : id, prio, interrupt function                                   */
/*  Out   : none                                                           */
/*  Return: TERR_NO_ERR / TERR_WRONG_PARAM                                 */
/***************************************************************************/
int tn_irq_install (TN_IRQ_ID   id,
                    TN_IRQ_PRIO prio,
                    void (*irq_func)(void))
{
   int           index;
   int           offset;
   unsigned long value;
   int           rc = TERR_NO_ERR;

   if( (id >= TN_MAX_IRQ_ID) || (prio >= TN_MAX_IRQ_PRIO))
   {
      rc = TERR_WRONG_PARAM;
   }
   else
   {
      /* 
       * The IRQTable will be used later to jump to the user function. 
       * See tn_port_sam3_irq.c
       */
      IRQTable[id] = irq_func;
      
      /*
       * For more information on exceptions and NVIC programming see 
       * Chap 5 Exceptions & Chap 8 Nested Vectored Interrupt Controller of 
       * the ARM Cortex-M3 Technical Reference Manual.
       * Specially  NVIC register description / Interrupt Priority Registers
       */
      index  = id / 4;
      offset = id & 0x03;
      
      value = InterruptPriorityRegistersTable[index];
      
      switch (offset)
      {
         case 0:
         {
            value &= ~0x000000FF;
            value |= (unsigned long)prio << 4;
            break;
         }
         case 1:
         {
            value &= ~0x0000FF00;
            value |= (unsigned long)prio << 12;
            break;
         }
         case 2:
         {
            value &= ~0x00FF0000;
            value |= (unsigned long)prio << 20;
            break;
         }
         case 3:
         {
            value &= ~0xFF000000;
            value |= (unsigned long)prio << 28;
            break;
         }
      }
      
      InterruptPriorityRegistersTable[id] = value;
   }
   
   return(rc);
} /* tn_irq_install */

#endif /* TN_ARCH_CM3 && TN_PORT_STM32 */
/*** EOF ***/
